%% FINDNTS - Tool to locate NT (no transmission) regions in audio
%
% |findNTs| is a Matlab script that automatically locates regions
% consisting of "no transmission" (NT) background noise in radio
% recordings.  The basic principle is that NT noise will be high
% energy noise with stationary characteristics (and low
% "periodicity", to help distinguish from strong voiced segments).
% These regions are identified without supervision: instead,
% cepstral and voicing features from 
% short frames are clustered, and the program looks for a popular, 
% narrow cluster with high energy.  This is then
% considered the NT cluster, and used to produce NT region labels.
% 
% Some receivers will apply "squelch" to such NT regions, where the
% output is cut to silence if the receiver decides there is no
% signal.  |findNTs| separately detects these regions (with a
% simple energy threshold with high temporal resolution) and
% excludes them from the modeling analysis (but also marks them as
% NT in the final output).
%
% Because the clustering depends on random initialization, the
% system repeats the entire estimation several times (5 by
% default), and keeps only the "best" result.  In the absence of
% label information, "best" means a combination of tightest NT
% cluster (smallest volume), and the best "modality" score, the
% ratio of the density of points at the cluster center to the
% density of points at around 2 SDs from the center (looking for a
% minimum in the point density surrounding the cluster).
%
% |findNTs| can write a label file consisting only of the
% identified NT regions, or it can read an existing label file and
% replace any existing NT labels with the new regions.  If input
% labels are provided and |-labelsformodel| is set, "best" is
% redefined to be the NT model with the greatest agreement with the
% provided labels. 
%
% The program offers a number of options to improve performance.
% Instead of processing files one by one, |findNTs| can be
% given a large collection of files.  It then clusters these files
% based on the similarity between their overall feature
% distributions, and builds models using sets of similar files.
% This helps deal with files that contain very little NT (or very
% little else).  An option also allows the program to search for a
% low energy NT mode, instead of preferring high energy (originally
% for channels where the output is "squelched" during NT; however,
% since v0.2, squelch is handled by a separate explicit squelch
% detector).
%
% |findNTs| can save the overall models it calculates, and then on
% a subsequent run reload these models instead of calculating new
% models, then relabel individual files using whichever model is
% most similar.

%% Example usage
%
% A single file is analyzed to identify the NT region.  A 3-column
% label file is written out with the ".fnt" extension

findNTs 21118_20110719_040600_10417_fsh-eng_A.flac

% "summary" info line includes
% SQ = total time identified as squelch (and removed); 
% LT = total time per feature calculation; 
% BM = index of 'best' model, 
% MDST = distance to best model, 
% FNTT = total time labeled NT by system
%
% Note: findNTs uses random initialization, so results can vary
% quite widely for individual files.

% Now include a graphical display of the model superimposed on the
% feature histogram (just try one model fit, else will pause on display)

findNTs -viewmodels 1 -tries 1 21118_20110719_040600_10417_fsh-eng_A.flac

%% Use with label files
%
% |findNTs| will optionally read an existing label file and report
% agreement/overlap with both NT and SP labels.

findNTs -labelpath . -viewmodels 1 -tries 1 21118_20110719_040600_10417_fsh-eng_A.flac

% Use -labelsformodel 1 to additionally use the labels to guide the
% model selection

findNTs -labelpath . -labelsformodel 1 21118_20110719_040600_10417_fsh-eng_A.flac

% Now the reports are contain more fields:  
% TT is the total time (in seconds) per the label file; 
% NTT is the total time for NT labels in that file;
% FNTNT is the amount of time marked NT by both the label file and
% the process;
% FNTNT% is the accuracy of the found NT regions against the NT
% labels;
% FNTSP, FNTSP% do the same, but comparing found NTs against SP
% labels. 


%% Use with multiple files
%
% When multiple input files are specified, they will be clustered
% and modeled in clusters.  We can also save the resulting models.
% Note that I fiddled with the criteria for clustering models here
% - models will be clustered if the (KL) distance between them is
% less than 4.0 x the 20th percentile of all the between-model
% distances in the set (excluding the self-distances).

findNTs -modelsout Amodels -labelpath A -audiopath A -audioext .flac ...
    -clstpcntile 0.2 -clstfact 4.0 ...
    21118_20110719_040600_10417_fsh-eng_A ...
    23218_20110811_185000_18744_A ...
    23241_20110814_142800_18766_A ...
    22212_20110818_035700_18830_A ...
    23219_20110811_192400_18803_A ...
    23247_20110815_091600_18807_A ...
    23200_20110808_200400_18735_A ...
    23221_20110811_235600_18790_A ...
    23248_20110816_144600_18788_A ...
    23209_20110809_101800_18760_A ...
    23226_20110812_055200_18777_A

% Instead of using a long list of filenames on the command line, you
% can also put them in a file and pass it in with -filelist <fname>.

%% Reusing saved models
%
% Instead of creating models from scratch each time, |findNTs| can
% load the models written by a previous run, then use whichever of
% these is the most similar to a particular file to decode it.
% This helps protect against chance incidence of bad models, since
% a set of good models can be created then reused.

findNTs -modelsin Amodels -labelpath A -audiopath A -audioext .flac ...
    21118_20110719_040600_10417_fsh-eng_A

%% Optional arguments
%
% This is the full range of arguments accepted:

findNTs -help 1


%% Installation
% 
% This package has been compiled for several targets 
% using the Matlab compiler.  You will also need 
% to download and install the Matlab Compiler Runtime (MCR) Installer. 
% Please see the table below:
%
% <html>
% <table border=1>
% <tr><th>Architecture</th><th>Compiled package</th><th>MCR Installer</th></tr>
% <tr><td>64 bit Linux</td>
% <td><a href="findNTs_GLNXA64.zip">findNTs_GLNXA64.zip</a></td>
% <td><a href="http://www.ee.columbia.edu/~dpwe/tmp/MCRInstaller_glnxa64.bin">Linux 64 bit MCR Installer</a></td></tr>
% <tr><td>64 bit MacOS</td>
% <td><a href="findNTs_MACI64.zip">findNTs_MACI64.zip</a></td>
% <td><a href="http://www.ee.columbia.edu/~dpwe/tmp/MCRInstaller.dmg">MACI64 MCR Installer</a></td></tr>
% </table></html>
% 
% The original Matlab code used to build this compiled target is 
% available at
% 
%   <http://labrosa.ee.columbia.edu/projects/findNTs/>
%
% All sources are in the package <findNTs-v@VER@.zip>.
%
% Feel free to contact me with any problems.
%

%% Changelog
%

% 2012-10-04 v0.35 - added -postout to optionally save raw NT posteriors
%
% 2012-07-25 v0.34 - minor tweak attempts to avoid 
%                    ill-condiitioned GMMs
%
% 2012-07-17 v0.33 - fixed bug in writing "patched" label files
%     where there was a gap between last input label and final
%     found NT.
%
% 2012-07-16 v0.32 - frames-per-file is reduced in proportion
%     to the number of files used in each cohort, so -nframes
%     dictates the total number of frames used for a model.
%     - small changes to how -verbose works (-verbose 2 is
%     verboser).
%
% 2012-07-10 v0.31  Identical results, but calculation of voicing
%     feature optimized so overall performance is about 2x faster 
%     (when features are not cached).
%
% 2012-07-05 v0.3 Improved performance with squelch
%   - various parameters tweaked to work better with files that may
%     contain very little high-energy NT because it has been
%     squelched.
%   - new "modality" measure (ratio of density at gaussian center
%     to density around mahalanobis radius = 2.0) used to choose
%     best model.
%   - added thresholds on modality measure and voicing feature to
%     entirely skip NT model labeling if it doesn't look like it
%     found a real NT mode (NT label are then only given by squelch
%     detection).
%   - various other tweaks to command line / UI (e.g., -filelist, 
%     -audiopath, -audioext).
%
%
% 2012-05-21 v0.2 Modified to handle new "squelchy" recordings
%   - initial desquelch followed by re-insertion of squelch gaps
%     into labels
%   - new autocorrelation voicing feature
%   - new -minklsep can reject models with poor separation between
%     NT and rest
%   - -justdesquelch bypasses modeling altogether (if all NT is squelched)
%   - some tweaks to gmm clustering (including merging bug fix)
%
% 2012-05-02 v0.1 Initial release
%

%% Acknowledgment
%
% This package includes viterbi_path.m and gaussian_prob.m from 
% <http://www.cs.ubc.ca/~murphyk/Software/HMM/hmm.html Kevin Murphy's HMM Toolbox>
%
% This work was supported by DARPA under the RATS program via a 
% subcontract from the SRI-led team SCENIC.  My work was on behalf 
% of ICSI.
%
% $Header: /Users/drspeech/data/RATS/code/findNTs/RCS/demo_findNTs.m,v 1.2 2012/05/02 14:36:47 dpwe Exp dpwe $
